<?php

namespace Bixo\Offplan\Http\Controllers;

use App\Http\Controllers\APIController as BaseController;
use Bixo\Offplan\Http\Requests\FloorplanRequest;
use Bixo\Offplan\Interfaces\FloorplanRepositoryInterface;
use Bixo\Offplan\Models\Floorplan;
use Bixo\Offplan\Forms\Floorplan as Form;

/**
 * APIController  class for floorplan.
 */
class FloorplanAPIController extends BaseController
{

    /**
     * Initialize floorplan resource controller.
     *
     * @param type FloorplanRepositoryInterface $floorplan
     *
     * @return null
     */
    public function __construct(FloorplanRepositoryInterface $floorplan)
    {
        parent::__construct();
        $this->repository = $floorplan;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Bixo\Offplan\Repositories\Criteria\FloorplanResourceCriteria::class);
    }

    /**
     * Display a list of floorplan.
     *
     * @return Response
     */
    public function index(FloorplanRequest $request)
    {
        return $this->repository
            ->setPresenter(\Bixo\Offplan\Repositories\Presenter\FloorplanPresenter::class)
            ->paginate();
    }

    /**
     * Display floorplan.
     *
     * @param Request $request
     * @param Model   $floorplan
     *
     * @return Response
     */
    public function show(FloorplanRequest $request, Floorplan $floorplan)
    {
        return $floorplan->setPresenter(\Bixo\Offplan\Repositories\Presenter\FloorplanListPresenter::class);
        ;
    }

    /**
     * Create new floorplan.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(FloorplanRequest $request)
    {
        try {
            $data              = $request->all();
            $data['user_id']   = user_id();
            $data['user_type'] = user_type();
            $data              = $this->repository->create($data);
            $message           = trans('messages.success.created', ['Module' => trans('offplan::floorplan.name')]);
            $code              = 204;
            $status            = 'success';
            $url               = guard_url('offplan/floorplan/' . $floorplan->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('offplan/floorplan');
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Update the floorplan.
     *
     * @param Request $request
     * @param Model   $floorplan
     *
     * @return Response
     */
    public function update(FloorplanRequest $request, Floorplan $floorplan)
    {
        try {
            $data = $request->all();

            $floorplan->update($data);
            $message = trans('messages.success.updated', ['Module' => trans('offplan::floorplan.name')]);
            $code    = 204;
            $status  = 'success';
            $url     = guard_url('offplan/floorplan/' . $floorplan->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('offplan/floorplan/' . $floorplan->getRouteKey());
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Remove the floorplan.
     *
     * @param Model   $floorplan
     *
     * @return Response
     */
    public function destroy(FloorplanRequest $request, Floorplan $floorplan)
    {
        try {
            $floorplan->delete();
            $message = trans('messages.success.deleted', ['Module' => trans('offplan::floorplan.name')]);
            $code    = 202;
            $status  = 'success';
            $url     = guard_url('offplan/floorplan/0');
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('offplan/floorplan/' . $floorplan->getRouteKey());
        }
        return compact('message', 'code', 'status', 'url');
    }

    /**
     * Return the form elements as json.
     *
     * @param String   $element
     *
     * @return json
     */
    public function form($element = 'fields')
    {
        $form = new Form();
        return $form->form($element, true);
    }

}
