<?php

namespace Bixo\Offplan\Http\Controllers;

use App\Http\Controllers\APIController as BaseController;
use Bixo\Offplan\Http\Requests\UnitRequest;
use Bixo\Offplan\Interfaces\UnitRepositoryInterface;
use Bixo\Offplan\Models\Unit;
use Bixo\Offplan\Forms\Unit as Form;

/**
 * APIController  class for unit.
 */
class UnitAPIController extends BaseController
{

    /**
     * Initialize unit resource controller.
     *
     * @param type UnitRepositoryInterface $unit
     *
     * @return null
     */
    public function __construct(UnitRepositoryInterface $unit)
    {
        parent::__construct();
        $this->repository = $unit;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Bixo\Offplan\Repositories\Criteria\UnitResourceCriteria::class);
    }

    /**
     * Display a list of unit.
     *
     * @return Response
     */
    public function index(UnitRequest $request)
    {
        return $this->repository
            ->setPresenter(\Bixo\Offplan\Repositories\Presenter\UnitPresenter::class)
            ->paginate();
    }

    /**
     * Display unit.
     *
     * @param Request $request
     * @param Model   $unit
     *
     * @return Response
     */
    public function show(UnitRequest $request, Unit $unit)
    {
        return $unit->setPresenter(\Bixo\Offplan\Repositories\Presenter\UnitListPresenter::class);
        ;
    }

    /**
     * Create new unit.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(UnitRequest $request)
    {
        try {
            $data              = $request->all();
            $data['user_id']   = user_id();
            $data['user_type'] = user_type();
            $data              = $this->repository->create($data);
            $message           = trans('messages.success.created', ['Module' => trans('offplan::unit.name')]);
            $code              = 204;
            $status            = 'success';
            $url               = guard_url('offplan/unit/' . $unit->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('offplan/unit');
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Update the unit.
     *
     * @param Request $request
     * @param Model   $unit
     *
     * @return Response
     */
    public function update(UnitRequest $request, Unit $unit)
    {
        try {
            $data = $request->all();

            $unit->update($data);
            $message = trans('messages.success.updated', ['Module' => trans('offplan::unit.name')]);
            $code    = 204;
            $status  = 'success';
            $url     = guard_url('offplan/unit/' . $unit->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('offplan/unit/' . $unit->getRouteKey());
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Remove the unit.
     *
     * @param Model   $unit
     *
     * @return Response
     */
    public function destroy(UnitRequest $request, Unit $unit)
    {
        try {
            $unit->delete();
            $message = trans('messages.success.deleted', ['Module' => trans('offplan::unit.name')]);
            $code    = 202;
            $status  = 'success';
            $url     = guard_url('offplan/unit/0');
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('offplan/unit/' . $unit->getRouteKey());
        }
        return compact('message', 'code', 'status', 'url');
    }

    /**
     * Return the form elements as json.
     *
     * @param String   $element
     *
     * @return json
     */
    public function form($element = 'fields')
    {
        $form = new Form();
        return $form->form($element, true);
    }

}
