<?php

namespace Bixo\Offplan\Policies;

use Litepie\User\Contracts\UserPolicy;
use Bixo\Offplan\Models\Floorplan;

class FloorplanPolicy
{

    /**
     * Determine if the given user can view the floorplan.
     *
     * @param UserPolicy $user
     * @param Floorplan $floorplan
     *
     * @return bool
     */
    public function view(UserPolicy $user, Floorplan $floorplan)
    {
        if ($user->canDo('offplan.floorplan.view') && $user->isAdmin()) {
            return true;
        }

        return $floorplan->user_id == user_id() && $floorplan->user_type == user_type();
    }

    /**
     * Determine if the given user can create a floorplan.
     *
     * @param UserPolicy $user
     * @param Floorplan $floorplan
     *
     * @return bool
     */
    public function create(UserPolicy $user)
    {
        return  $user->canDo('offplan.floorplan.create');
    }

    /**
     * Determine if the given user can update the given floorplan.
     *
     * @param UserPolicy $user
     * @param Floorplan $floorplan
     *
     * @return bool
     */
    public function update(UserPolicy $user, Floorplan $floorplan)
    {
        if ($user->canDo('offplan.floorplan.edit') && $user->isAdmin()) {
            return true;
        }

        return $floorplan->user_id == user_id() && $floorplan->user_type == user_type();
    }

    /**
     * Determine if the given user can delete the given floorplan.
     *
     * @param UserPolicy $user
     * @param Floorplan $floorplan
     *
     * @return bool
     */
    public function destroy(UserPolicy $user, Floorplan $floorplan)
    {
        return $floorplan->user_id == user_id() && $floorplan->user_type == user_type();
    }

    /**
     * Determine if the given user can verify the given floorplan.
     *
     * @param UserPolicy $user
     * @param Floorplan $floorplan
     *
     * @return bool
     */
    public function verify(UserPolicy $user, Floorplan $floorplan)
    {
        if ($user->canDo('offplan.floorplan.verify')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the given user can approve the given floorplan.
     *
     * @param UserPolicy $user
     * @param Floorplan $floorplan
     *
     * @return bool
     */
    public function approve(UserPolicy $user, Floorplan $floorplan)
    {
        if ($user->canDo('offplan.floorplan.approve')) {
            return true;
        }

        return false;
    }

    /**
     * Determine if the user can perform a given action ve.
     *
     * @param [type] $user    [description]
     * @param [type] $ability [description]
     *
     * @return [type] [description]
     */
    public function before($user, $ability)
    {
        if ($user->isSuperuser()) {
            return true;
        }
    }
}
