<?php

namespace Bixo\Offplan\Providers;

use Illuminate\Support\ServiceProvider;

class OffplanServiceProvider extends ServiceProvider
{
    /**
     * Indicates if loading of the provider is deferred.
     *
     * @var bool
     */
    protected $defer = false;

    /**
     * Bootstrap the application events.
     *
     * @return void
     */
    public function boot()
    {
        // Load view
        $this->loadViewsFrom(__DIR__ . '/../../resources/views', 'offplan');

        // Load translation
        $this->loadTranslationsFrom(__DIR__ . '/../../resources/lang', 'offplan');

        // Load migrations
        $this->loadMigrationsFrom(__DIR__ . '/../../database/migrations');

        // Call pblish redources function
        $this->publishResources();

    }

    /**
     * Register the service provider.
     *
     * @return void
     */
    public function register()
    {
        $this->mergeConfig();
        $this->registerOffplan();
        $this->registerFacade();
        $this->registerBindings();
        //$this->registerCommands();
    }


    /**
     * Register the application bindings.
     *
     * @return void
     */
    protected function registerOffplan()
    {
        $this->app->bind('offplan', function($app) {
            return new Offplan($app);
        });
    }

    /**
     * Register the vault facade without the user having to add it to the app.php file.
     *
     * @return void
     */
    public function registerFacade() {
        $this->app->booting(function()
        {
            $loader = \Illuminate\Foundation\AliasLoader::getInstance();
            $loader->alias('Offplan', 'Lavalite\Offplan\Facades\Offplan');
        });
    }

    /**
     * Register bindings for the provider.
     *
     * @return void
     */
    public function registerBindings() {
        // Bind facade
        $this->app->bind('bixo.offplan', function ($app) {
            return $this->app->make('Bixo\Offplan\Offplan');
        });

        // Bind Floorplan to repository
        $this->app->bind(
            'Bixo\Offplan\Interfaces\FloorplanRepositoryInterface',
            \Bixo\Offplan\Repositories\Eloquent\FloorplanRepository::class
        );        // Bind Project to repository
        $this->app->bind(
            'Bixo\Offplan\Interfaces\ProjectRepositoryInterface',
            \Bixo\Offplan\Repositories\Eloquent\ProjectRepository::class
        );        // Bind Unit to repository
        $this->app->bind(
            'Bixo\Offplan\Interfaces\UnitRepositoryInterface',
            \Bixo\Offplan\Repositories\Eloquent\UnitRepository::class
        );        // Bind Booking to repository
        $this->app->bind(
            'Bixo\Offplan\Interfaces\BookingRepositoryInterface',
            \Bixo\Offplan\Repositories\Eloquent\BookingRepository::class
        );

        $this->app->register(\Bixo\Offplan\Providers\AuthServiceProvider::class);
        
        $this->app->register(\Bixo\Offplan\Providers\RouteServiceProvider::class);
            }

    /**
     * Merges user's and offplan's configs.
     *
     * @return void
     */
    protected function mergeConfig()
    {
        $this->mergeConfigFrom(
            __DIR__ . '/../../config/config.php', 'bixo.offplan'
        );
    }

    /**
     * Register scaffolding command
     */
    protected function registerCommands()
    {
        if ($this->app->runningInConsole()) {
            $this->commands([
                Commands\MakeOffplan::class,
            ]);
        }
    }
    /**
     * Get the services provided by the provider.
     *
     * @return array
     */
    public function provides()
    {
        return ['bixo.offplan'];
    }

    /**
     * Publish resources.
     *
     * @return void
     */
    private function publishResources()
    {
        // Publish configuration file
        $this->publishes([__DIR__ . '/../../config/config.php' => config_path('bixo/offplan.php')], 'config');

        // Publish admin view
        $this->publishes([__DIR__ . '/../../resources/views' => base_path('resources/views/vendor/offplan')], 'view');

        // Publish language files
        $this->publishes([__DIR__ . '/../../resources/lang' => base_path('resources/lang/vendor/offplan')], 'lang');

        // Publish public files and assets.
        $this->publishes([__DIR__ . '/public/' => public_path('/')], 'public');
    }
}
