<?php

namespace Bixo\Offplan\Providers;

use Illuminate\Foundation\Support\Providers\RouteServiceProvider as ServiceProvider;
use Illuminate\Routing\Router;
use Bixo\Offplan\Models\Offplan;
use Request;
use Route;

class RouteServiceProvider extends ServiceProvider
{
    /**
     * This namespace is applied to the controller routes in your routes file.
     *
     * In addition, it is set as the URL generator's root namespace.
     *
     * @var string
     */
    protected $namespace = 'Bixo\Offplan\Http\Controllers';

    /**
     * Define your route model bindings, pattern filters, etc.
     *
     * @param   \Illuminate\Routing\Router  $router
     * @return void
     */
    public function boot()
    {
        parent::boot();

        if (Request::is('*/offplan/floorplan/*')) {
            Route::bind('floorplan', function ($floorplan) {
                $floorplanrepo = $this->app->make('Bixo\Offplan\Interfaces\FloorplanRepositoryInterface');
                return $floorplanrepo->findorNew($floorplan);
            });
        }
        if (Request::is('*/offplan/project/*')) {
            Route::bind('project', function ($project) {
                $projectrepo = $this->app->make('Bixo\Offplan\Interfaces\ProjectRepositoryInterface');
                return $projectrepo->findorNew($project);
            });
        }
        if (Request::is('*/offplan/unit/*')) {
            Route::bind('unit', function ($unit) {
                $unitrepo = $this->app->make('Bixo\Offplan\Interfaces\UnitRepositoryInterface');
                return $unitrepo->findorNew($unit);
            });
        }
        if (Request::is('*/offplan/booking/*')) {
            Route::bind('booking', function ($booking) {
                $bookingrepo = $this->app->make('Bixo\Offplan\Interfaces\BookingRepositoryInterface');
                return $bookingrepo->findorNew($booking);
            });
        }

    }

    /**
     * Define the routes for the package.
     *
     * @return void
     */
    public function map()
    {
        $this->mapWebRoutes();

        $this->mapApiRoutes();
    }

    /**
     * Define the "web" routes for the package.
     *
     * These routes all receive session state, CSRF protection, etc.
     *
     * @return void
     */
    protected function mapWebRoutes()
    {   
        Route::group([
            'middleware' => 'web',
            'namespace'  => $this->namespace,
        ], function ($router) {
            require (__DIR__ . '/../../routes/web.php');
        });
    }

    /**
     * Define the "api" routes for the package.
     *
     * These routes are typically stateless.
     *
     * @return void
     */
    protected function mapApiRoutes()
    {
        Route::group([
            'middleware' => 'api',
            'namespace'  => $this->namespace,
            'prefix'     => 'api',
        ], function ($router) {
            require (__DIR__ . '/../../routes/api.php');
        });
    }

}
